@@ -21,7 +21,8 @@ return 0;
 sub open_last_command
 {
 local ($fh, $user) = @_;
-open($fh, "last $user |");
+local $quser = defined($user) && length($user) ? " ".quotemeta($user) : "";
+open($fh, "last$quser |");
 }
 
 # read_last_line(handle)
@@ -43,12 +44,5 @@ while(1) {
 	}
 }
 
-# encrypt_password(password)
-sub encrypt_password
-{
-local $salt = chr(int(rand(26))+65) . chr(int(rand(26))+65);
-return crypt($_[0], $salt);
-}
-
 1;
 
