@@ -18,6 +18,14 @@ $access{'upgrade'} = 0 if (&is_readonly_mode());	# too hard to fake
 &foreign_require("acl");
 %text = ( %webmin::text, %text );
 
+if (!defined($gconfig{'noselfwebminup'})) {
+	$access{'upgrade'} = 0
+		if (&webmin::has_repos());
+	}
+else {
+	$access{'upgrade'} = !$gconfig{'noselfwebminup'};
+	}
+
 $usermin_miniserv_config = "$config{'usermin_dir'}/miniserv.conf";
 $usermin_config = "$config{'usermin_dir'}/config";
 
@@ -57,26 +65,35 @@ sub put_usermin_miniserv_config
 &write_file($usermin_miniserv_config, \%usermin_miniserv_config_cache);
 }
 
-=head2 get_usermin_version
+=head2 get_usermin_version([format_dev_version], [include_release])
 
 Returns the version number of Usermin on this system.
 
 =cut
 sub get_usermin_version
 {
-my ($ui_format_dev) = @_;
+my ($ui_format_dev, $inc_release_version) = @_;
 local %miniserv;
 &get_usermin_miniserv_config(\%miniserv);
 open(VERSION, "<$miniserv{'root'}/version");
 local $version = <VERSION>;
 close(VERSION);
 $version =~ s/\r|\n//g;
+my $release_version = "";
+# Usermin minor version
+if ($inc_release_version) {
+	my $usermin_version_release = read_file_contents("$miniserv{'root'}/release") || "";
+	$usermin_version_release =~ s/\r|\n//g;
+	$release_version = "-".$usermin_version_release
+	    if ($usermin_version_release > 1);
+	}
+
 # Format dev version nicely
 if ($ui_format_dev && length($version) == 13) {
-	return substr($version, 0, 5) . "." . substr($version, 5, 5 - 1) . "." . substr($version, 5 * 2 - 1);
+	return substr($version, 0, 5) . "." . substr($version, 5, 5 - 1) . "." . substr($version, 5 * 2 - 1) . $release_version;
 	}
 else {
-	return $version;
+	return $version . $release_version;
 	}
 }
 
@@ -403,6 +420,7 @@ if (&is_readonly_mode()) {
 open(MFILE, "<".$file);
 read(MFILE, $two, 2);
 close(MFILE);
+local $qfile = quotemeta($file);
 if ($two eq "\037\235") {
 	if (!&has_command("uncompress")) {
 		unlink($file) if ($need_unlink);
@@ -410,13 +428,15 @@ if ($two eq "\037\235") {
 		}
 	local $temp = $file =~ /\/([^\/]+)\.Z/i ? &transname("$1")
 						: &transname();
-	local $out = `uncompress -c "$file" 2>&1 >$temp`;
+	local $qtemp = quotemeta($temp);
+	local $out = `uncompress -c $qfile 2>&1 >$qtemp`;
 	unlink($file) if ($need_unlink);
 	if ($?) {
 		unlink($temp);
 		return &text('install_ecomp2', $out);
 		}
 	$file = $temp;
+	$qfile = quotemeta($file);
 	$need_unlink = 1;
 	}
 elsif ($two eq "\037\213") {
@@ -426,13 +446,15 @@ elsif ($two eq "\037\213") {
 		}
 	local $temp = $file =~ /\/([^\/]+)\.gz/i ? &transname("$1")
 						 : &transname();
-	local $out = `gunzip -c "$file" 2>&1 >$temp`;
+	local $qtemp = quotemeta($temp);
+	local $out = `gunzip -c $qfile 2>&1 >$qtemp`;
 	unlink($file) if ($need_unlink);
 	if ($?) {
 		unlink($temp);
 		return &text('install_egzip2', $out);
 		}
 	$file = $temp;
+	$qfile = quotemeta($file);
 	$need_unlink = 1;
 	}
 
@@ -445,7 +467,7 @@ open(TYPE, "<../install-type");
 chop($type = <TYPE>);
 close(TYPE);
 if ($type eq 'rpm' && $file =~ /\.rpm$/i &&
-    ($out = `rpm -qp $file 2>/dev/null`)) {
+    ($out = `rpm -qp $qfile 2>/dev/null`)) {
 	# Looks like an RPM of some kind, hopefully an RPM usermin module
 	# or theme
 	local ($out, %minfo, %tinfo);
@@ -454,7 +476,7 @@ if ($type eq 'rpm' && $file =~ /\.rpm$/i &&
 		return $text{'install_erpm'};
 		}
 	$redirect_to = $name = $2;
-	$out = &backquote_logged("rpm -U \"$file\" 2>&1");
+	$out = &backquote_logged("rpm -Uv $qfile 2>&1");
 	if ($?) {
 		unlink($file) if ($need_unlink);
 		return &text('install_eirpm', "<tt>$out</tt>");
@@ -498,7 +520,7 @@ else {
 	# Check if this is a valid module (a tar file of multiple module or
 	# theme directories)
 	local (%mods, %hasfile);
-	local $tar = `tar tf "$file" 2>&1`;
+	local $tar = `tar tf $qfile 2>&1`;
 	if ($?) {
 		unlink($file) if ($need_unlink);
 		return &text('install_etar', $tar);
@@ -531,7 +553,7 @@ else {
 		next if (!$hasfile{$m,"module.info"});
 		push(@realmods, $m);
 		local %minfo;
-		system("cd $tmpdir ; tar xf \"$file\" $m/module.info ./$m/module.info >/dev/null 2>&1");
+		system("cd $tmpdir ; tar xf $qfile $m/module.info ./$m/module.info >/dev/null 2>&1");
 		if (!&read_file("$tmpdir/$m/module.info", \%minfo)) {
 			$err = &text('install_einfo', "<tt>$m</tt>");
 			}
@@ -583,7 +605,7 @@ else {
 		}
 
 	# Extract all the modules and update perl path and ownership
-	local $out = `cd $miniserv{'root'} ; tar xf "$file" 2>&1 >/dev/null`;
+	local $out = `cd $miniserv{'root'} ; tar xf $qfile 2>&1 >/dev/null`;
 	if ($?) {
 		unlink($file) if ($need_unlink);
 		return &text('install_eextract', $out);
@@ -783,6 +805,7 @@ sub delete_usermin_module
 {
 local $m = $_[0];
 return undef if (!$m);
+local $qm = quotemeta($m);
 local %minfo = &get_usermin_module_info($m);
 %minfo = &get_usermin_theme_info($m) if (!%minfo);
 return undef if (!%minfo);
@@ -815,7 +838,7 @@ else {
 		@lst = stat("$miniserv{'root'}/$l");
 		if (-l "$miniserv{'root'}/$l" && $lst[1] == $mst[1]) {
 			&unlink_logged("$miniserv{'root'}/$l");
-			&system_logged("rm -rf $config{'usermin_dir'}/$l");
+			&system_logged("rm -rf ".quotemeta("$config{'usermin_dir'}/$l"));
 			push(@clones, $l);
 			}
 		}
@@ -831,7 +854,7 @@ else {
 			   "<tt>$mdir</tt>", $size);
 	if ($type eq 'rpm') {
 		# This module was installed from an RPM .. rpm -e it
-		&system_logged("rpm -e ubm-$m");
+		&system_logged("rpm -e ubm-$qm");
 		}
 	else {
 		# Module was installed from a .wbm file .. just rm it
@@ -855,22 +878,12 @@ sub flush_modules_cache
 
 =head2 stop_usermin
 
-Kills the running Usermin server process, returning undef on success or an
-error message on failure.
+Kills the running Usermin server process. Return value is always undef.
 
 =cut
 sub stop_usermin
 {
-local %miniserv;
-&get_usermin_miniserv_config(\%miniserv);
-local $pid;
-if (open(PID, "<".$miniserv{'pidfile'}) && ($pid = int(<PID>))) {
-	&kill_logged('TERM', $pid) || return &text('stop_ekill', $!);
-	close(PID);
-	}
-else {
-	return $text{'stop_efile'};
-	}
+&system_logged("$config{'usermin_dir'}/stop >/dev/null 2>&1 </dev/null");
 return undef;
 }
 
@@ -1064,4 +1077,3 @@ chmod(0755, $wrapper);
 }
 
 1;
-
